unit uFRename;
{$INCLUDE 'FRename3.inc'}

interface

implementation

uses
  Windows, SysUtils, Classes, Variants,
  {$INCLUDE 'DelphiFarUnits.inc'} ,
  uLog, uSystem, uMessages, uFRenameConsts,
  uOptions, uOptionsUI,
  uFileList;

type
  TPlugin = class(TFarPlugin)
  private
    fPluginInfoAvailable: boolean;
    fPluginInfo: TPluginInfo;
    fOptions: TOptions;
    function GetOptions: TOptions;
  protected
    function PluginID: TGUID; override;
    function GetPluginTitle(out Description, Author: string): string; override;
    function GetPluginInfo(out Info: TPluginInfo): boolean; override;
    function Execute(OpenFrom: DWORD; GUID: PGUID; Data: TIntPtr): THandle; override;
    function GetCommonString(ID: TFarPlugin.TCommonString): string; override;
    function PluginConfig(GUID: PGUID; MenuItem: TIntPtr): boolean; override;
    procedure DoLog(const Msg: string); override;
    procedure RefreshOptions;
    property Options: TOptions read GetOptions;
    function ExecuteFromPanel: boolean;
    function ExecuteFromFileList(const Directory: string; const Files: TStrings): boolean;
    function ProcessMacroCall(const Params: array of Variant): boolean;
  public
    constructor Create;
    destructor Destroy; override;
  end;

const
  PLUGIN_TITLE = 'FRename3';
  PLUGIN_DESCRIPTION = 'Files Rename 3';
  PLUGIN_AUTHOR = 'Pepak, http://www.pepak.net';

{ TPlugin }

constructor TPlugin.Create;
begin
  inherited;
  fPluginInfoAvailable := False;
  fOptions := nil;
end;

destructor TPlugin.Destroy;
begin
  FreeAndNil(fOptions);
  if fPluginInfoAvailable then
  begin
    DeletePluginInfo(fPluginInfo);
  end;
  inherited;
end;

procedure TPlugin.DoLog(const Msg: string);
begin
  {$IFDEF DEBUG_LOGS}
  uLog.Log(Msg);
  {$ENDIF}
end;

function TPlugin.Execute(OpenFrom: DWORD; GUID: PGUID; Data: TIntPtr): THandle;
var
  List: TFileList;
begin
  Result := 0;
  RefreshOptions;
  if OpenFrom = {$IFDEF FAR3_UP} OPEN_FROMMACRO_ {$ELSE} OPEN_FROMMACRO {$ENDIF} then
  begin
    if ProcessMacroCall(GetMacroParams(Data)) then
      Result := 1;
    Exit;
  end;
  if OpenFrom = OPEN_EDITOR then
  begin
    if TFileList.FindByEditorID(-1, List) then
      List.ShowEditorMenu;
  end
  else
  begin
    if not ExecuteFromPanel then
      ShowWarning(GetMsg(MFailedToLoadFileList));
  end;
end;

function TPlugin.ExecuteFromFileList(const Directory: string; const Files: TStrings): boolean;
var
  List: TFileList;
begin
  Result := False;
  List := TFileList.Create(Options);
  try
    if not List.LoadFromFileList(Directory, Files) then
      Exit;
    if not List.OpenInEditor then
      Exit;
    List := nil; // This prevents the editor from being destroyed
    Result := True;
  finally
    FreeAndNil(List);
  end;
end;

function TPlugin.ExecuteFromPanel: boolean;
var
  List: TFileList;
begin
  Result := False;
  List := TFileList.Create(Options);
  try
    if not List.LoadFromPanel(True) then
      Exit;
    if not List.OpenInEditor then
      Exit;
    List := nil; // This prevents the editor from being destroyed
    Result := True;
  finally
    FreeAndNil(List);
  end;
end;

function TPlugin.GetCommonString(ID: TFarPlugin.TCommonString): string;
begin
  case ID of
    TFarPlugin.TCommonString.MTitle:
      Result := GetMsg(MTitle);
    TFarPlugin.TCommonString.MExceptionOccurredIn:
      Result := GetMsg(MExceptionOccurredIn);
    TFarPlugin.TCommonString.MOk:
      Result := GetMsg(MOk);
    TFarPlugin.TCommonString.MCancel:
      Result := GetMsg(MCancel);
    TFarPlugin.TCommonString.MYes:
      Result := GetMsg(MYes);
    TFarPlugin.TCommonString.MNo:
      Result := GetMsg(MNo);
    else
      Result := inherited GetCommonString(ID);
  end;
end;

function TPlugin.GetOptions: TOptions;
begin
  if fOptions = nil then
  begin
    fOptions := TOptions.Create;
    fOptions.Load;
  end;
  Result := fOptions;
end;

function TPlugin.GetPluginInfo(out Info: TPluginInfo): boolean;
var
  fMainMenu: TFarPlugin.TFarPluginMenu;
  fConfigMenu: TFarPlugin.TFarPluginMenu;
  fDiskMenu: TFarPlugin.TFarPluginMenu;
begin
  if not fPluginInfoAvailable then
  begin
    SetLength(fMainMenu, 1);
    fMainMenu[0].Title := GetMsg(MTitle);
    fMainMenu[0].ID := MENU_PANEL_GUID;
    SetLength(fConfigMenu, 1);
    fConfigMenu[0].Title := GetMsg(MTitle);
    fConfigMenu[0].ID := MENU_OPTIONS_GUID;
    SetLength(fDiskMenu, 0);
    Self.fPluginInfo := CreatePluginInfo(
      PF_EDITOR,
      fMainMenu, fConfigMenu, fDiskMenu,
      ''
    );
    fPluginInfoAvailable := True;
  end;
  Info := Self.fPluginInfo;
  Result := True;
end;

function TPlugin.GetPluginTitle(out Description, Author: string): string;
begin
  Result := PLUGIN_TITLE;
  Description := PLUGIN_DESCRIPTION;
  Author := PLUGIN_AUTHOR;
end;

function TPlugin.PluginConfig(GUID: PGUID; MenuItem: TIntPtr): boolean;
var
  Dialog: TOptionsDialog;
begin
  Result := False;
  RefreshOptions;
  Dialog := TOptionsDialog.Create;
  try
    Dialog.Options := Options;
    if Dialog.Execute then
    begin
      Options.Save;
      Result := True;
    end;
  finally
    FreeAndNil(Dialog);
  end;
end;

function TPlugin.PluginID: TGUID;
begin
  Result := PLUGIN_GUID;
end;

function TPlugin.ProcessMacroCall(const Params: array of Variant): boolean;

  function StringListFromParam(const Param: Variant): TStringList;
  var
    ParamStr: string;
    ParamStrLen: integer;
    ItemStart: integer;
    i: integer;
  begin
    Result := TStringList.Create;
    try
      if VarIsArray(Param) then
      begin
        for i := VarArrayLowBound(Param, 1) to VarArrayHighBound(Param, 1) do
          Result.Add(Param[i]);
      end
      else
      begin
        ParamStr := Param;
        ParamStrLen := Length(ParamStr);
        ItemStart := 1;
        for i := 1 to ParamStrLen do
        begin
          if (ParamStr[i] = #13) or (ParamStr[i] = #10) then
          begin
            if (ParamStr[i] = #10) and (i > 1) and (ParamStr[Pred(i)] = #13) then
              // Already processed
            else
            begin
              Result.Add(Copy(ParamStr, ItemStart, i-1));
              ItemStart := Succ(i);
            end;
          end;
        end;
        if ItemStart < ParamStrLen then
        begin
          Result.Add(Copy(ParamStr, ItemStart, MaxInt));
        end;
      end;
    except
      FreeAndNil(Result);
      Raise;
    end;
  end;

var
  Command: string;
  List: TStringList;
  Editor: TFileList;
begin
  Result := False;
  if Length(Params) > 0 then
  begin
    Command := Params[0];
    if Command = 'panel' then
    begin
      Result := ExecuteFromPanel;
      Exit;
    end;
    if Command = 'files' then
    begin
      if Length(Params) <> 3 then
        raise EFarPluginError.Create(GetMsg(MInvalidMacroParameter));
      List := StringListFromParam(Params[2]);
      try
        Result := ExecuteFromFileList(Params[1], List);
      finally
        FreeAndNil(List);
      end;
      Exit;
    end;
    if Command = 'iseditor' then
    begin
      if TFileList.FindByEditorID(-1, Editor) then
        Result := True;
      Exit;
    end;
    if Command = 'cursortofilename' then
    begin
      if TFileList.FindByEditorID(-1, Editor) then
        if Editor.JumpToDestinationFileName then
          Result := True;
      Exit;
    end;
    if Command = 'selectfilenames' then
    begin
      if TFileList.FindByEditorID(-1, Editor) then
        if Editor.SelectDestinationFilenames then
          Result := True;
      Exit;
    end;
  end;
end;

procedure TPlugin.RefreshOptions;
begin
  FreeAndNil(fOptions);
end;

exports
  ProcessEditorEventW,
  ProcessEditorInputW;

initialization
  TFarPlugin.InitInstance(TPlugin.Create);

end.
