unit uOptionsUI;
{$INCLUDE 'FRename3.inc'}

interface

uses
  SysUtils,
  {$INCLUDE 'DelphiFarUnits.inc'} ,
  uOptions;

type
  TOptionsDialog = class(TOkCancelDialog)
  private
    fOptions: TOptions;
    fCheckTinyHelp: TFarDialog.TCheckbox;
    fCheckSelectRenamedFiles: TFarDialog.TCheckbox;
  protected
    procedure Clear; override;
    procedure CreateUserControls; override;
    procedure Resize(ScreenWidth, ScreenHeight: integer); override;
    procedure UpdateSize(var Width, Height: integer); override;
    property CheckTinyHelp: TFarDialog.TCheckbox read fCheckTinyHelp;
    property CheckSelectRenamedFiles: TFarDialog.TCheckbox read fCheckSelectRenamedFiles;
  public
    constructor Create;
    destructor Destroy; override;
    function Execute: boolean;
    property Options: TOptions read fOptions write fOptions;
  end;

implementation

uses
  ShellApi, Windows,
  uMessages, uFRenameConsts;

{ TOptionsDialog }

procedure TOptionsDialog.Clear;
begin
  inherited;
  fCheckTinyHelp := nil;
  fCheckSelectRenamedFiles := nil;
end;

constructor TOptionsDialog.Create;
begin
  inherited Create(DIALOG_OPTIONS_GUID);
end;

procedure TOptionsDialog.CreateUserControls;
begin
  inherited;
  fCheckTinyHelp := AddCheckbox(0, 0, GetMsg(MTinyHelpOption), False);
  fCheckSelectRenamedFiles := AddCheckbox(0, 0, GetMsg(MSelectRenamedFilesOption), False);
end;

destructor TOptionsDialog.Destroy;
begin
  inherited;
end;

function TOptionsDialog.Execute: boolean;
begin
  Result := False;
  CheckTinyHelp.Checked := Options.TinyHelp;
  CheckSelectRenamedFiles.Checked := Options.SelectRenamedFiles;
  Resize(Screen.Width, Screen.Height);
  if BuildDialog(Width, Height, 0) then
  begin
    if inherited Execute then
    begin
      Options.TinyHelp := CheckTinyHelp.Checked;
      Options.SelectRenamedFiles := CheckSelectRenamedFiles.Checked;
      Result := True;
    end;
  end;
end;

procedure TOptionsDialog.Resize(ScreenWidth, ScreenHeight: integer);
begin
  inherited;
  CheckTinyHelp.AlignTo(Frame, 2, 1, -4, 0);
  CheckSelectRenamedFiles.AlignTo(CheckTinyHelp, 0, 1, 0, 0);
end;

procedure TOptionsDialog.UpdateSize(var Width, Height: integer);
begin
  inherited;
  Height := 8;
end;

end.
